# Usage

You can use the submarine.jar to simulate / visualize games. Start it with `java -jar submarine.jar`. It supports several arguments and config files.

# Quick Start

Open the provided `submarine.txt` config file and change `map`, `ship_command` and `submarine_command` to the desired values. Run `java -jar submarine.jar` in a console, or `java -jar submarine.jar -vis` if you want to see more than text. To replay a recorded game run `java -jar submarine.jar -replay log/game.txt`.

# Arguments

The server accepts the following arguments:

 - `-config file`: Loads the given config file. This argument can be passed several times with different files. For configurations occurring more than once the last value will be used.
 - `-log config`: The configuration file for the logger.
 - `-replay file`: Shows the replay instead of a live game. Implies `-vis`.
 - `-tournament file`: Plays the given tournament file.
 - `-vis`: Start the server with visualization.

# Configuration File

If no configuration file is specified using an argument, the file `submarine.txt` is loaded. The file contains one config item per line, first the key and after a space the value. The last line contains `---`.

In the configuration files the following keys are accepted:

 - `map`: The map to use. Can be either an image or a text file.
 - `log`: Configuration file for the logger.
 - `replay`: Shows the replay instead of a live game. Implies `-vis`.
 - `ship_count`: Number of ships.
 - `submarine_count`: Number of submarines.
 - `radius_move`: Radius objects are allowed to move.
 - `radius_view`: Radius a submarine can see.
 - `radius_attack`: Radius a submarine can attack.
 - `time_limit`: Time each bot has available. Using a chess clock like limit.
 - `round_limit`: The maximal number of rounds to be played.
 - `ship_command`: Command to start the ship AI. This can be an executable command (use `./command` for local executables) with arguments (e.g. `python bot.py`). Use `:AI`, `:AII`, `:SUB1`, or `:SUB2` for bots included in the server.
 - `submarine_command`: Command to start the submarine AI. Same rules as for `ship_command` apply.
 - `ship_name`: Name of the ship AI.
 - `submarine_name`: Name of the submarine AI.
 - `sleep_time`: Time in ms between two steps if the visualization runs.
 - `tournament`: Play a tournament.
 - `tournamentfile`: The file to use for the tournament.
 - `visualization`: Show the visualization.

The following keys are accepted as well but are not important for everyday use:

 - `vis_ship_radius`
 - `vis_shot_radius`
 - `vis_debug_radius`
 - `vis_show_debug`
 - `replay_velo_base`
 - `replay_velo_exp`

# Log Config

If no configuration file is specified using an argument, the file `logconfig.txt` is loaded. In the config file for each log channel three flags (0 or 1) and one string can be specified, whether to print the channel on stdout, stderr or file. If file is true, the channel will be written to the file specified by the string. If it is false, the string is ignored, but it can't be empty. Several outputs can be enabled at the same time. The first line of the file contains one String, the folder in which to save the logs.

The following channels are available, in the order as in the config file:

 - `Debug`: Debug messages of the server.
 - `Game`: Log file of the game, used for replay.
 - `Behaviour`: Behaviour of a bot (e.g. bot crashed).
 - `External Debug`: The communication between the server and the external bots.
 - `Process Err`: Things an external bot writes to its stderr.
 - `Tournament`: Results of a tournament.

# Tournament file

A tournament file contains information about a tournament. The first line contains two integers `n m`, the number of games and the number of players. `n` lines follow, each containing `map submarine_count ship_count move_radius view_radius attack_radius`. `2m` lines follow, the names and the commands for the players (first a name, then a command, alternating). Each player will play against each other player (once as ship and once as submarine) and this is repeated for all given games.

# Map format

A map can be specified using text or an image.
In a text file the first line contains two integers `w h`, the size of the map. `h` lines with `w` chars each follow.
For an image the dimensions of the image is used as the size. The content of the image is the map.

The following field types are available:

 - `Land`: `#`, `#000000`
 - `Water`: `.`, `#0000FF`
 - `Start`: `s`, `#00FF00`
 - `Destination`: `d`, `#FF0000`