#include <bits/stdc++.h>

using namespace std;
using pairint=pair<int, int>;
using vc=vector<char>;
using vvc=vector<vc>;
using vb=vector<bool>;
using vbb=vector<vb>;
using vpairint=vector<pairint>;

struct vessel{
	int x, y;
	bool destroyed, arrived;
};

// all methods
void submarine();
void ship();
void resetUsed();
void initPositions(char character, int count);
void getMatchingPositions(char character, bool equals, vpairint & possibles);
pairint chooseRandom(vpairint & possibles);
int dist2(pairint & a, pairint & b);
int dist2(vessel & a, pairint & b);
int dist2(int x1, int y1, int x2, int y2);

int side, shipCount, subCount, radiusMove, radiusView, radiusAttack;
int width, height;
vvc gameMap;
vbb used;
vector<vessel> vessels;

// play as a submarine
void submarine(){
	// choose starting positions
	initPositions('d', subCount);
	while(true){
		// read input (visible ships)
		int visibleShipCount; cin >> visibleShipCount;
		if(visibleShipCount < 0){
		    break;
		}
		vpairint visibleShips(visibleShipCount);
		for(int i = 0; i < visibleShipCount; i++){
			cin >> visibleShips[i].first >> visibleShips[i].second;
		}
		// update the map of used positions
		resetUsed();
		for(vessel & v : vessels){
			used[v.x][v.y] = true;
		}
		// play for each vessel
		for(vessel & v : vessels){
			if(v.destroyed || v.arrived){
				continue;
			}
			// check if a ship is within reach to attack
			bool found = false;
			for(pairint & visibleShip : visibleShips){
				if(dist2(v, visibleShip) <= radiusAttack){
					// output attack
					cout << "A " << visibleShip.first << " " << visibleShip.second << "\n";
					found = true;
					break;
				}
			}
			if(found){
				continue;
			}
			// if no ship to attack is within reach move to a random nearby position
			vpairint possibles;
			getMatchingPositions('#', false, possibles);
			vpairint possibleMoves;
			// filter matching positions by distance
			for(auto & possible : possibles){
				if(dist2(v, possible) <= radiusMove){
					possibleMoves.push_back(possible);
				}
			}
			// move vessel
			if(!possibleMoves.empty()){
				auto possible = chooseRandom(possibleMoves);
				used[v.x][v.y] = false;
				used[possible.first][possible.second] = true;
				v.x = possible.first;
				v.y = possible.second;
			}
			// output move
			cout << "M " << v.x << " " << v.y << "\n";
		}
		cout << flush;
	}
}

// play as a ship
void ship(){
	// choose starting positions
	initPositions('s', shipCount);
	while(true){
		// read input (sunken ships)
		int sunkenShipsCount; cin >> sunkenShipsCount;
		if(sunkenShipsCount < 0){
		    break;
		}
		// mark sunken ships as destroyed
		for(int i = 0; i < sunkenShipsCount; i++){
			pairint ship, submarine; cin >> ship.first >> ship.second >> submarine.first >> submarine.second;
			for(vessel & v : vessels){
				if(v.x == ship.first && v.y == ship.second){
					v.destroyed = true;
				}
			}
		}
		// update the map of used positions
		resetUsed();
		for(vessel & v : vessels){
			used[v.x][v.y] = true;
		}
		// play for each vessel
		for(vessel & v : vessels){
			if(v.destroyed || v.arrived){
				continue;
			}
			// move to a random nearby position
			vpairint possibles;
			getMatchingPositions('#', false, possibles);
			vpairint possibleMoves;
			// filter matching positions by distance
			for(auto & possible : possibles){
				if(dist2(v, possible) <= radiusMove){
					possibleMoves.push_back(possible);
				}
			}
			// move vessel
			if(!possibleMoves.empty()){
				auto possible = chooseRandom(possibleMoves);
				used[v.x][v.y] = false;
				used[possible.first][possible.second] = true;
				v.x = possible.first;
				v.y = possible.second;
			}
			// output move
			cout << "M " << v.x << " " << v.y << "\n";
			if(gameMap[v.x][v.y] == 'd'){
				v.arrived = true;
			}
		}
		cout << flush;
	}
}

void resetUsed(){
	for(int x = 0; x < width; x++){
		fill(used[x].begin(), used[x].end(), false);
	}
}

// choose count positions where the field is equals to character and print them
void initPositions(char character, int count){
	for(int i = 0; i < count; i++){
		vpairint possibles;
		getMatchingPositions(character, true, possibles);
		pairint possible = chooseRandom(possibles);
		used[possible.first][possible.second] = true;
		vessels.push_back({possible.first, possible.second, false, false});
		cout << possible.first << " " << possible.second << "\n";
	}
	cout << flush;
}

// get a list of positions where the field is equals or not equals to character
void getMatchingPositions(char character, bool equals, vpairint & possibles){
	for(int y = 0; y < height; y++){
		for(int x = 0; x < width; x++){
			if(((equals && gameMap[x][y] == character) || (!equals && gameMap[x][y] != character)) && !used[x][y]){
				possibles.push_back({x, y});
			}
		}
	}
}

// choose a random position from the given possible positions
pairint chooseRandom(vpairint & possibles){
	if(possibles.empty()){
		return {-1, -1};
	}
	return possibles[rand() % possibles.size()];
}

// helper methods to calculate the distance between two points

int dist2(pairint & a, pairint & b){
	return dist2(a.first, a.second, b.first, b.second);
}

int dist2(vessel & a, pairint & b){
	return dist2(a.x, a.y, b.first, b.second);
}

int dist2(int x1, int y1, int x2, int y2){
	return (x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2);
}

// main method
int main(){
	srand(42);
	cin >> side >> shipCount >> subCount >> radiusMove >> radiusView >> radiusAttack;
	cin >> width >> height;
	gameMap.resize(width, vc(height));
	used.resize(width, vb(height));
	// read game map
	for(int y = 0; y < height; y++){
		for(int x = 0; x < width; x++){
			cin >> gameMap[x][y];
		}
	}
	// play the game
	if(side){
		submarine();
	} else {
		ship();
	}
	return 0;
}
