/*===================================
	Creative Task Submarine 2016
=====================================*/
#include <bits/stdc++.h>

using namespace std;
// different types
typedef long long ll;
typedef long double ld;


// some global constants
ll width, height;				// width and height of the global field
vector<vector<char> > field;	// global field
ll rmove, rsight, rattack;		// different radii, movement, sight and attack
ll nrBoats, nrSubmarines;		// amount of ships and submarines
bool isSubmarine;				// defines role of player
// checks if the position is on the field
bool inRange(ll x, ll y){
	if(x < 0 || x >= width){return false;}
	if(y < 0 || y >= height){return false;}
	return true;
}
// checks if the position we want to move to is in the allowed movement range
bool inDist(ll x, ll y){
	if(x*x + y*y > rmove){
		return false;
	}
	return true;
}
// a struct for points (convenient for coding)
struct pt{
	ll x,y;		// x and y coordinate
	pt(){}
	// some constructors
	pt(pt &a, pt &b){
		x = b.x - a.x;
		y = b.y - a.y;
	}
	pt(ll _x, ll _y){
		x = _x; y = _y;
	}
	// returns if the points are identical or not
	bool match(pt &a){
		if(a.x == x && a.y == y){
			return true;
		}else{
			return false;
		}
	}
	// add a delta x and y to our position, will not check if the move is valid
	void add(ll dx,ll dy){
		x += dx; y += dy;
	}
	// returns distance^2 to a point
	ll distance(pt &a){
		return (x-a.x)*(x-a.x)+(y-a.y)*(y-a.y);
	}
	// prints the point
	void pr(){
		cout << x << " " << y << endl;
	}
};
// we store the ending and starting positions so that we can override the global map
vector<pt> startingPositions;
vector<pt> endingPositions;
// checks if a given position is an end position
bool isEndPos(pt &a){
	for(int i=0;i<(int)endingPositions.size();i++){
		if(a.match(endingPositions[i])){return true;}
	}
	return false;
}

// a struct for the ships that we must coordinate to the destination
struct boat{
	pt position; // position of the boat
	bool atEnd=false; // is true if reached an end point
	// some constructors
	boat(){}
	boat(ll _x, ll _y){
		position = pt(_x, _y);
	}
	boat(pt a){
		position = a;
	}
	// checks if two boats are the same
	bool match(pt a){
		if(position.match(a)){
			return true;
		}
		return false;
	}
	// prints the boat
	void pr(){
		position.pr();
	}
	// will be called to make a turn == > is RANDOM
	void turn(){
		ll dist = 2*rmove;
		ll dx = rand()%(dist+1) - rmove;
		ll dy = rand()%(dist+1) - rmove;
		ll counter = 0;
		// checks that the boat makes a valid move
		while(inRange(position.x+dx,position.y+dy)==false || inDist(dx,dy)==false
				|| field[position.y+dy][position.x+dx] == 'X' || field[position.y +dy][position.x+dx]=='#'){
			dx = rand()%(dist+1) -rmove;
			dy = rand()%(dist+1) -rmove;
			if(counter > 100000){dx=0;dy=0;} // if we too many moves we will just no make a move
			counter++;
		}
		field[position.y][position.x] = '.';
		position.add(dx,dy);
		cout << "M ";
		pr();
		if(isEndPos(position)){
			atEnd = true;
			return;
		}
		field[position.y][position.x] = 'X';
	}
};
// a vector of all boats - if the bot commands the submarines it will store the boats it sees in this vector
vector<boat> boats;
// struct for a submarine
struct submarine{
	pt position;	// position of submarine
	// some constructors
	submarine(){}
	submarine(ll x,ll y){
		position = pt(x,y);
	}
	submarine(pt a){
		position = a;
	}
	// print the submarine
	void pr(){
		position.pr();
	}
	// will be called to make a turn == > is RANDOM (the bot will shoot every boat that he can)
	void turn(){
		// attack if see a boat
		for(int i=0;i<(int)boats.size();i++){
			if(position.distance(boats[i].position) <= rattack){
				cout << "A "; boats[i].pr();	// attack the boat and erase it (it will be dead)
				boats.erase(boats.begin()+i);
				return;
			}
		}
		// otherwise make a move
		ll dist = 2*rmove;
		ll dx = rand()%(dist+1) - rmove;
		ll dy = rand()%(dist+1) - rmove;
		ll counter = 0;
		// check if our random makes a valid move
		while(inRange(position.x+dx,position.y+dy)==false || inDist(dx,dy)==false
				|| field[position.y+dy][position.x+dx]=='X' || field[position.y +dy][position.x+dx]=='#'){
			dx = rand()%(dist+1) -rmove;
			dy = rand()%(dist+1) -rmove;
			if(counter > 100000){dx=0;dy=0;} // if we too many moves we will just no make a move
			counter++;
		}
		field[position.y][position.x] = '.';
		position.add(dx,dy);
		cout << "M ";
		pr();
		field[position.y][position.x] = 'X';
	}
};
// a vector for all the submarines
vector<submarine> submarines;
struct game{
	bool finished = false;
	game(){}
	void begin(){
		// read input
		cin >> isSubmarine >> nrBoats >> nrSubmarines;
		cin >> rmove >> rsight >> rattack;
		// width and height
		cin >> width >> height;
		field = vector<vector<char> > (height,vector<char>(width,'.'));
		for(int i = 0;i < height;i++){
			for(int j = 0; j< width; j++){
				cin >> field[i][j];
				if(field[i][j] == 'd'){
					endingPositions.push_back(pt(j,i));
				}
				if(field[i][j]== 's'){
					startingPositions.push_back(pt(j,i));
				}
			}
		}
		// set our boats or submarines
		init();
	}
	void init(){
		// choose a starting position for submarines or boats ==> RANDOM
		random_shuffle(startingPositions.begin(),startingPositions.end());
		random_shuffle(endingPositions.begin(),endingPositions.end());

		if(isSubmarine){
			// place submarines now
			for(int i=0;i<nrSubmarines;i++){
				submarines.push_back(submarine(endingPositions[i]));
				field[endingPositions[i].y][endingPositions[i].x] = 'X';
			}
		}else{
			// place boats somewhere
			for(int i=0;i<nrBoats;i++){
				boats.push_back(boat(startingPositions[i]));
				field[startingPositions[i].y][startingPositions[i].x] = 'X';
			}
		}
	}
	void input(){
		ll number;
		ll x,y,u,v;
		if(isSubmarine){
			// amount of boats we see
			cin >> number;
			boats.clear();
			for(int i=0;i<number;i++){
				// cordinates of uboot
				cin >> x >> y;
				boats.push_back(boat(x,y));
			}
		}else{
			// amount of boats that were destroyed and where
			cin >> number;
			submarines.clear();
			for(int i=0;i<number;i++){
				cin >> x >> y >> u >> v;
				submarines.push_back(submarine(x,y));
				// delte boat from boats
				for(int i=0;i<(int)boats.size();i++){
					if(boats[i].match(pt(x,y))){
						boats.erase(boats.begin()+i);
						nrBoats--;
						break;
					}
				}
			}
		}
		if(number < 0){
			finished = true;
		}
	}
	void turn(){
		// make a turn for each boat or submarine
		if(isSubmarine){
			for(int i=0;i<(int)submarines.size();i++){
				submarines[i].turn();
			}
		}else{
			for(int i=0;i<(int)boats.size();i++){
				boats[i].turn();
			}
			// delete boat if reached end
			for(int i=0;i<(int)boats.size();i++){
				if(boats[i].atEnd){
					boats.erase(boats.begin()+i);
					i--;
				}
			}
		}
	}
	void printPositions(){
		// print all positions of the submarines or boats
		if(isSubmarine){
			for(int i=0;i<(int)submarines.size();i++){
				submarines[i].pr();
			}
		}else{
			for(int i=0;i<(int)boats.size();i++){
				boats[i].pr();
			}
		}
	}
};

int main(){
	game G;
	G.begin();
	// print all position at the beginning
	G.printPositions();
	// game loop
	while(true){
		G.input(); // read the input
		if(G.finished){
			break;
		}
		G.turn(); // make a turn
	}
	return 0;
}